#!/bin/bash

LOG_FUNC_FILE="$(dirname "$0")"/log.sh
. "${LOG_FUNC_FILE}" || { echo "[$(date -d today +'%Y-%m-%d %H:%M:%S')] [ERROR]: Failed to load ${LOG_FUNC_FILE}. [${SCRIPT_NAME}:${LINENO}]($$)"; exit 1;}

# ---------------------------------------------------------------------------- #
#                               全局变量定义                                   #
# ---------------------------------------------------------------------------- #
declare ntpServerIP=""
declare floatIPs=""
declare manager_ip_list=""
declare business_ip_list=""
declare log_file=""
declare -a ntp_server_array=""
declare -a float_ip_array=""

# ---------------------------------------------------------------------------- #
#                          ntp同步时间函数定义                                 #
# ---------------------------------------------------------------------------- #
if [ $UID -ne 0 ] && [ -n "${INSTALL_SUDO_PATH}" ]; then
    CMD_SYNC_TIME="sudo ${INSTALL_SUDO_PATH} sync_time"
else
    CMD_SYNC_TIME="sync_time"
fi

################################################################################
# Function: sntp_sync_time
# Description: 使用sntp命令同步ntp服务器的时间
# Parameter:
#   input:ntp服务器ip
# Return: 0 -- success; not 0 -- failure
################################################################################
sntp_sync_time()
{
    local ntp_server_ip=$1
    if [ -z "$ntp_server_ip" ]; then
        log "[ERROR_DEBUG]" "${LINENO}" "ntp_server_ip is empty."
        return 1
    fi
    #sntp协议的时间戳最大支持2036年2月7日6:28:15(UTC)，如果系统时间年份大于等于2036年，就把系统时间改为2035年01月01日(UTC)，确保sntp强制同步成功。
    if [ $(date '+%Y') -ge 2036 ]; then
        log "[DEBUG]" "${LINENO}" "system time is going to change to 20350101(UTC) ."
        date -u -s '20350101' >> "${log_file}" 2>&1
    fi

    sntp -v -P no -r ${ntp_server_ip} >> "${log_file}" 2>&1
    if [ $? -ne 0 ]; then
        log "[ERROR_DEBUG]" "${LINENO}" "sntp -P no -r ${ntp_server_ip} failed."
        
        sntp -S ${ntp_server_ip} >> "${log_file}" 2>&1
        if [ $? -ne 0 ]; then
            log "[ERROR_DEBUG]" "${LINENO}" "sntp -S ${ntp_server_ip} failed."
            return 1
        else
            log "[DEBUG]" "${LINENO}" "sntp -S ${ntp_server_ip} successfully."
            return 0
        fi
    else
        log "[DEBUG]" "${LINENO}" "sntp -P no -r ${ntp_server_ip} successfully."
        return 0
    fi
}

################################################################################
# Function: ntpdate_sync_time
# Description: 使用ntpdate命令同步ntp服务器的时间
# Parameter:
#   input:ntp服务器ip
# Return: 0 -- success; not 0 -- failure
################################################################################
ntpdate_sync_time()
{
    local ntp_server_ip=$1
    if [ -z "$ntp_server_ip" ]; then
        log "[ERROR_DEBUG]" "${LINENO}" "ntp_server_ip is empty."
        return 1
    fi
    #等待3秒钟接收ntp服务器的相应并强制同步时间
    ntpdate -t 3 ${ntp_server_ip} >> "${log_file}" 2>&1
    if [ $? -ne 0 ]; then
        log "[ERROR_DEBUG]" "${LINENO}" "ntpdate ${ntp_server_ip} failed."
        return 1
    fi

    log "[DEBUG]" "${LINENO}" "ntpdate ${ntp_server_ip} successfully."
    return 0
}

################################################################################
# Function: sync_time
# Description: 同步ntp服务器时间
# Parameter:
#   input:ntp服务器ip
# Return: 0 -- success; not 0 -- failure
################################################################################
function sync_time()
{
    local ntp_server_ip=$1
    log "[DEBUG]" "${LINENO}" "sync_time"

    if [ -z "$ntp_server_ip" ]; then
        log "[ERROR_DEBUG]" "${LINENO}" "ntp_server_ip is empty."
        return 1
    fi

    which sntp > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        sntp_sync_time ${ntp_server_ip}
        return $?
    fi

    which ntpdate > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        ntpdate_sync_time ${ntp_server_ip}
        return $?
    fi

    log "[ERROR_DEBUG]" "${LINENO}" "can't find sntp or ntpdate programe."
    return 1
}

################################################################################
# Function: get_ntpServer_array
# Description: Gets the ntpServer IP address from the configuration file
# Return: 0 -- success; not 0 -- failure
################################################################################
function get_ntpServer_array()
{
    if [ -z "${ntpServerIP}" ]; then
        log "[INFO]" "${LINENO}" "Ntp server IP is empty, so skip to verify it."
        return 0
    fi
    
    ntp_server_array=( $(echo "${ntpServerIP}" | tr "," " ") )
    local ntp_server_num=${#ntp_server_array[@]}
    if [ "${ntp_server_num}" -gt 2 ]; then
        log "[ERROR_DEBUG]" "${LINENO}" "The ntp server number $ntp_server_num is more than 2."
        return 1
    fi
}

################################################################################
# Function: check_ntpServer_reachable
# Description: Check whether the IP address of ntpServer is reachable
# Return: 0 -- success; not 0 -- failure
################################################################################
function check_ntpServer_reachable()
{
    local tmpIP=""
    for tmpIP in "${ntp_server_array[@]}" ; do
        ping -c 1 "${tmpIP}"
        if [ $? -ne 0 ]; then
            ping -c 1 -6 "${tmpIP}"
            if [ $? -ne 0 ]; then
                log "[ERROR_DEBUG]" "${LINENO}" "ping ntp server ${tmpIP} failed."
                return 1
            fi
        fi
    done
}

################################################################################
# Function: check_ntpServer_ip
# Description: Check whether the ntpServer IP is already in the cluster
# Return: 0 -- success; not 0 -- failure
################################################################################
function check_ntpServer_ip()
{
    float_ip_array=( $(echo "${floatIPs}" | tr "," " ") )
    
    local tmpIP=""
    for tmpIP in "${ntp_server_array[@]}" ; do
        local ipLine=""
        #检查ntpServer的IP地址是否与om或者ws的浮动ip相同
        for ipLine in "${float_ip_array[@]}" ; do
            if [ "${tmpIP}" = "${ipLine}" ] ; then
                log "[ERROR_DEBUG]" "${LINENO}" "The ntp server ip ${tmpIP} can not be same with the float ip in cluster."
                return 1
            fi
        done
        
        #检查ntpServer的IP地址是否与集群中节点的管理ip或者业务ip相同
        ipLine=""
        for ipLine in $(cat "$manager_ip_list" "$business_ip_list" | sort | uniq); do
            ipLine="$(echo "${ipLine}" | sed "s/[[:space:]]//g")"
            if [ "${tmpIP}" = "${ipLine}" ]; then
                log "[ERROR_DEBUG]" "${LINENO}" "The ntp server ip ${tmpIP} can not be same with node ip in cluster."
                return 1
            fi
        done
    done
}

################################################################################
# Function: check_ntp_availability
# Description: check ntp availability by sync time with ntp
# Return: 0 -- success; not 0 -- failure
################################################################################
function check_ntp_availability()
{
    local tmpIP=""
    for tmpIP in "${ntp_server_array[@]}" ; do
        ${CMD_SYNC_TIME} ${tmpIP} ${log_file}
        if [ $? -ne 0 ]; then
            log "[ERROR_DEBUG]" "${LINENO}" "sync time from ntp server ${tmpIP} failed."
            return 1
        fi
    done
}

################################################################################
# Function: check_ntpServer
# Description: main function to check ntp server
# Parameter:
#   input: oms installation config file, manager ip list flie,
#          business ip list file
# Return: 0 -- success; not 0 -- failure
################################################################################
check_ntpServer()
{
    ntpServerIP="$1"
    floatIPs="$2"
    manager_ip_list="$3"
    business_ip_list="$4"
    log_file="$5"
    
    get_ntpServer_array
    if [ $? -ne 0 ]; then
        return 1
    elif [ ${#ntp_server_array[@]} -eq 0 -o -z "${ntpServerIP}" ]; then
        #ntp_server_array为空，会在安装时设置为默认值，直接返回
        return 0
    fi
    
    #检查ntpServer的ip是否可达
    check_ntpServer_reachable
    if [ $? -ne 0 ]; then
        return 1
    fi
    
    #检查ntpServer的IP地址是否与当前集群中的IP地址冲突（包括浮动IP）
    check_ntpServer_ip
    if [ $? -ne 0 ]; then
        return 1
    fi
    
    #检查ntpServer的可用性：同步一次时间
    check_ntp_availability
    if [ $? -ne 0 ]; then
        return 1
    fi
}

function main()
{
    log "[INFO]" "${LINENO}" "localhost: enter checking ntp server."
    check_ntpServer "$@"
    if [ $? -ne 0 ]; then
        log "[ERROR]" "${LINENO}" "localhost: check ntp server failed."
        return 1
    else
        log "[INFO]" "${LINENO}" "localhost: end checking ntp server."
        return 0
    fi
}

main "$@"