#!/bin/bash

################################################
#
# 获取系统中缺失的rpm包
#
################################################
function get_missing_rpms()
{
    local missing_rpms="$1"    #缺失的rpm包列表
    local rpms_dir="$2"        #rpm包目录
    local symbol="$3"          #节点标识 -- ip地址
    local target_dir=""        #保存获取的rpm包的目录
    local line=""              #文件每行的内容
    local file=""              #rpm包文件名

    debug "[${missing_rpms}:${rpms_dir}:${symbol}]Enter [${FUNCNAME}:${LINENO}]"

    #入参检查
    if [ ! -f "$missing_rpms" ]; then
        error "[${FUNCNAME}:${LINENO}]" "${missing_rpms} is not a file"
        return 1
    fi

    if [ -z "$symbol" ]; then
        error "[${FUNCNAME}:${LINENO}]" "symbol is null"
        return 1
    fi

    #检查rpms_dir
    check_rpms_dir "$rpms_dir"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "check_rpms_dir failed"
        return 1
    fi

    target_dir="${g_ctrl_space}/${symbol}_rpms_dir"

    #先删除目录 再创建目录 保证目录存在 且为空目录
    rm "${target_dir}" -rf
    mkdir "$target_dir"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "mkdir $target_dir failed"
        return 1
    fi

    while read line
    do
        #排除非字母或数字开头的行
        if [ -z "$(echo "$line" | grep "^\**[[:alnum:]]")" ]; then
            continue
        fi

        #获取rpm包文件
        file="$(get_rpm_file "$rpms_dir" "$line" "0")"
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "Don't find [${line}] rpm package in $rpms_dir for ${symbol}"
            return 1
        fi

        #把找到的rpm包拷贝至目标目录
        cp "$file" "$target_dir" -af
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "cp $file $target_dir -af failed"
            return 1
        fi

    done < $missing_rpms

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

###################################################
#
# 功能:  获取对应的软件包路径
#
###################################################
function get_pkgs_path()
{
    local osname="$1"
    local type=""
    local line=""
    local pkg_dir=""

    debug "[${osname}]Enter [${FUNCNAME}:${LINENO}]"

    #逐项匹配
    while read line
    do
        if [ -z "$line" ]; then
            continue
        fi

        #获取对应的目录
        type="$(echo $line | awk -F':' '{print $1}')"
        if [ "${type}" == "${osname}" ]; then
            pkg_dir="$(echo $line | awk -F':' '{print $2}')"
            break
        fi

    done < <(echo "$g_pkgs_dir" | tr ';' '\n')

    if [ -z "$pkg_dir" ]; then
        error "[${FUNCNAME}:${LINENO}]" "get pkg_dir failed"
        return 1
    fi

    debug "[${pkg_dir}]Leave [${FUNCNAME}:${LINENO}]"

    echo "$pkg_dir"

    return 0
}

###################################################
#
# 功能:  从文件名中获取OS名称
#
###################################################
function get_osname_from_filename()
{
    local file="$1"     #文件名
    local osname=""

    debug "[${file}]Enter [${FUNCNAME}:${LINENO}]"

    #判断匹配到的文件个数 必须为 1
    if [ $(ls -l ${file} | wc -l) -ne 1 ]; then
        error "[${FUNCNAME}:${LINENO}]" "[$(ls -l ${file})] Multiple files"
        return 1
    fi

    #获取 对应的 OS版本
    osname=$(basename "$(ls ${file})" | awk -F'_' '{print $1}')
    if [ -z "$osname" ]; then
        error "[${FUNCNAME}:${LINENO}]" "get osname failed from [${file}]"
        return 1
    fi

    debug "[${osname}]Leave [${FUNCNAME}:${LINENO}]"
    echo "${osname}"

    return 0
}

###################################################
#
# 功能:  在远端安装ssh命令
#
###################################################
function add_packages_r()
{
    local ipaddr="$1"  #主机名或ip地址
    local ret=0        #返回码
    local i=0          #循环变量
    local osname=""    #os名称
    local pkg_dir=""   #rpm包目录
    local missing_lst=""
    local rpm_names=""
    local add_pkg_path="${g_preinstall_path}/function/add_packages"
    local path_r="${g_work_space}/add_packages/"

    debug "Enter [${FUNCNAME}:${LINENO}]"

    #判断是否需要添加
    if [ "${g_add_pkg}" -eq 0 ]; then
        info "[${FUNCNAME}:${LINENO}]" "Don't add packages."
        return 0
    fi

    #入参检查
    if [ -z "$ipaddr" ]; then
        error "[${FUNCNAME}:${LINENO}]" "Parameter error. ipaddr is null."
        return 1
    fi

    #把 脚本和配置文件 拷贝至远端服务器
    cp_file_to_r "$ipaddr" "240" "${add_pkg_path}" "${g_work_space}" "0" > /dev/null
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "cp [${add_pkg_path}] to $ipaddr failed"
        return 1
    fi

    #在远端服务器上执行命令
    exec_cmd_r "$ipaddr" "360" "bash ${g_check_rpm_r} \"${ipaddr}\" \"${g_debug}\""
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "[${ipaddr}] get miss rpms script failed."
        return 1
    fi

    #把列表从远端 拷贝至本机
    cp_file_from_r "$ipaddr" "240" "${path_r}/*_${ipaddr}_missing-rpms.lst" "${g_ctrl_space}" "0" > /dev/null
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Copy [${path_r}/*_${ipaddr}_missing-rpms.lst] from $ipaddr failed"
        return 1
    fi

    #从文件名中获取OS名称
    osname="$(get_osname_from_filename "${g_ctrl_space}/*_${ipaddr}_missing-rpms.lst")"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "get_osname_from_filename failed[${g_ctrl_space}/*_${ipaddr}_missing-rpms.lst]"
        return 1
    fi

    #获取对应的软件包路径
    pkg_dir="$(get_pkgs_path "${osname}")"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "No path matches ${osname} for ${ipaddr}, please check g_pkgs_dir in preinstall.ini"
        return 1
    fi

    missing_lst="${g_ctrl_space}/${osname}_${ipaddr}_missing-rpms.lst"

    #若为新增os适配补包，将osList.ini文件拷贝到/usr/local/osAdapter/下
    if [ -f "${g_mian_path}/adaptOK" ]; then
        #创建/usr/local/osAdapter/目录
        exec_cmd_r "$ipaddr" "600" "${CMD_MKDIR_P} \"/usr/local/osAdapter\"" "1"
        ret=$?
        if [ $ret -ne 0 ]; then
            if [ $ret -eq 129 ];then
                error "[${FUNCNAME}:${LINENO}]" "Executing timeout, please check network."
                return 1 
            fi
            error "[${FUNCNAME}:${LINENO}]" "mkdir  /usr/local/osAdapter/ failed"
            return 1
        fi

        cp_file_to_r "$ipaddr" "240" "${g_mian_path}/osList.ini" "${g_work_space}" "0" > /dev/null
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "Copy [osList.ini] to $ipaddr failed"
            return 1
        fi
        exec_cmd_r "$ipaddr" "600" "${CMD_PREINSTALL_CP} \"${g_work_space}/osList.ini\" \"/usr/local/osAdapter/\"" "1"
        ret=$?
        if [ $ret -ne 0 ]; then
            if [ $ret -eq 129 ];then
                error "[${FUNCNAME}:${LINENO}]" "Executing timeout, please check network."
                return 1 
            fi
            error "[${FUNCNAME}:${LINENO}]" "chmod 755 /usr/local/osAdapter/ failed"
            return 1
        fi

        #修改/usr/local/osAdapter/目录权限为755
        exec_cmd_r "$ipaddr" "600" "${CMD_PREINSTALL_CHMOD_R} 755 \"/usr/local/osAdapter/\"" "1"
        ret=$?
        if [ $ret -ne 0 ]; then
            if [ $ret -eq 129 ];then
                error "[${FUNCNAME}:${LINENO}]" "Executing timeout, please check network."
                return 1 
            fi
            error "[${FUNCNAME}:${LINENO}]" "chmod 755 /usr/local/osAdapter/ failed"
            return 1
        fi    
    fi

    #判断是否需要增加rpm包
    if [ -f "${missing_lst}" -a -z "$(cat ${missing_lst} | grep "^\**[[:alnum:]]")" ]; then
        info "[${FUNCNAME}:${LINENO}]" "Do not lack rpms in ${ipaddr}"
        return 0
    fi

    #获取系统中缺失的rpm包
    get_missing_rpms "$missing_lst" "$pkg_dir" "$ipaddr"
    if [ $? -ne 0 ]; then
        info "[${FUNCNAME}:${LINENO}]" "get_missing_rpms failed [${missing_lst}] [${pkg_dir}] [${ipaddr}]"
        return 1
    fi

    #把缺失的rpm拷贝至远端服务器
    cp_file_to_r "$ipaddr" "1200" "${g_ctrl_space}/${ipaddr}_rpms_dir" "${g_work_space}" "0" 
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Copy [${g_ctrl_space}/${ipaddr}_rpms_dir] to $ipaddr failed"
        return 1
    fi

    #在远端服务器上执行命令
    exec_cmd_r "$ipaddr" "600" "${CMD_RPM_INSTALL} \"${g_work_space}/${ipaddr}_rpms_dir/*.rpm\"" "1"
    ret=$?
    if [ $ret -ne 0 ]; then
        if [ $ret -eq 129 ];then
            error "[${FUNCNAME}:${LINENO}]" "Executing timeout, please check network."
            return 1 
        fi
        rpm_names=$(cat ${missing_lst} | grep -v "^#" | grep -v "^[[:space:]]*$" | tr '[:space:]' ' ')
        error "[${FUNCNAME}:${LINENO}]" "Install [ ${rpm_names}] on ${ipaddr} failed. May be rpm packages version is inconsistent, please check."
        return 1
    fi

    #清理中间文件
    rm -rf "${g_ctrl_space}/${ipaddr}_rpms_dir"
    exec_cmd_r "$ipaddr" "300" "rm ${g_work_space}/* -rf" "0"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "rm ${g_work_space}/* -rf failed on ${ipaddr}"
        return 1
    fi
    #麒麟-V10-fedora的os系统补包后拉起rngd
    idLike=$(grep "^ID_LIKE=" "/etc/os-release" | awk -F "\"" '{print $2}')
    if [ "${idLike}" = "fedora" ]; then
        exec_cmd_r "$ipaddr" "30" "systemctl restart rngd" "0"
    fi

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

function add_libssl_libcrypto_link()
{
    local osname=""
    local ipaddr="$1"  #主机名或ip地址

    debug "Enter [${FUNCNAME}:${LINENO}] for node ${ipaddr}."

    osname=$(_get_osname)
    echo ${osname} | grep "suse" > /dev/null 2>&1
    if [ $? -eq 0 ] && [ `uname -p` == 'x86_64' ]; then
        if [ $(id -u) -eq 0 ] ; then
            cmd="if [ ! -e /usr/lib64/libssl.so.0.9.8  ]; then \
                    libssl_file=\`find  /lib64/ /usr/lib64/ -maxdepth 1 -type f -name 'libssl.so*' | sort -r | awk 'NR==1'\`; \
                    ln -s \$libssl_file /usr/lib64/libssl.so.0.9.8 >/dev/null 2>&1; \
                fi; \
                if [ ! -e /usr/lib64/libcrypto.so.0.9.8  ]; then \
                    libcrypto_file=\`find  /lib64/ /usr/lib64/ -maxdepth 1 -type f -name 'libcrypto.so*' | sort -r | awk 'NR==1'\`; \
                    ln -s \$libcrypto_file /usr/lib64/libcrypto.so.0.9.8 >/dev/null 2>&1; \
                fi"
        else
            cmd="sudo ${INSTALL_SUDO_PATH} create_soft_link_for_libssl_libcrypto"
        fi

        exec_cmd_r "$ipaddr" "600" "${cmd}" "1"
        ret=$?
        if [ $ret -ne 0 ]; then
            if [ $ret -eq 129 ];then
                error "[${FUNCNAME}:${LINENO}]" "Executing timeout, please check network."
                return 1
            fi
            error "[${FUNCNAME}:${LINENO}]" "fail to modify libssl and libcrypto version, please check."
            return 1
        fi
    fi

    echo ${osname} | grep -Eqi "euleros|openEuler|bclinux|kylin-V10-SP|uos|fusionos|kylinSecOS|LinxOS"
    if [ $? -eq 0 ]
    then
        local boost_vers=(1.66.0 1.53.0)
        for boost_ver in ${boost_vers[@]}
        do
            execute_add_libssl_libcrypto $boost_ver $ipaddr
            ret=$?
            if [ $ret -ne 0 ]; then
                if [ $ret -eq 129 ];then
                    error "[${FUNCNAME}:${LINENO}]" "Executing timeout, please check network."
                    return 1
                fi
                error "[${FUNCNAME}:${LINENO}]" "fail to modify boost version, please check."
                return 1
            fi
        done
    fi

    local os_type=$(get_osname_r "${ipaddr}")
    #openEuler 22.03版本，以及euler-2.11及之后版本需要创建liblber-2.4.so.2，libldap-2.4.so.2对应的软链接
    if [ "${os_type}" == "openEuler-22.03" ] || [[ "${os_type}" =~ "openEuler-23" ]] || [[ "${os_type}" =~ euleros-2.0sp1[1-9] ]] || [[ "${os_type}" =~ LinxOS-[1-9] ]];then
        create_soft_link_for_openldap $ipaddr
        ret=$?
        if [ $ret -ne 0 ]; then
            if [ $ret -eq 129 ];then
                error "[${FUNCNAME}:${LINENO}]" "Executing create_soft_link_for_openldap timeout, please check network."
                return 1
            fi
            error "[${FUNCNAME}:${LINENO}]" "fail to create softlink for openldap, please check."
            return 1
        fi

        create_soft_link_for_ldlinux
        ret=$?
        if [ $ret -ne 0 ]; then
            if [ $ret -eq 129 ];then
                error "[${FUNCNAME}:${LINENO}]" "Executing create_soft_link_for_ldlinux timeout, please check network."
                return 1
            fi
            error "[${FUNCNAME}:${LINENO}]" "fail to create softlink for ld-linux, please check."
            return 1
        fi
    fi

    return 0
}



function create_soft_link_for_openldap()
{
    local ipaddr="$1"
    if [ $(id -u) -eq 0 ] ; then
        cmd="if [ ! -e /usr/lib64/liblber-2.4.so.2 ]; then \
                liblber_file=\`find  /lib64/ /usr/lib64/ -maxdepth 1 -type f -name 'liblber.so*' | sort -r | awk 'NR==1'\`; \
                ln -s \$liblber_file /usr/lib64/liblber-2.4.so.2 >/dev/null 2>&1; \
            fi; \
            if [ ! -e /usr/lib64/libldap-2.4.so.2 ]; then \
                libldap_file=\`find  /lib64/ /usr/lib64/ -maxdepth 1 -type f -name 'libldap.so*' | sort -r | awk 'NR==1'\`; \
                ln -s \$libldap_file /usr/lib64/libldap-2.4.so.2 >/dev/null 2>&1; \
            fi"
    else
        cmd="sudo ${INSTALL_SUDO_PATH} create_soft_link_for_openldap"
    fi

    exec_cmd_r "$ipaddr" "600" "${cmd}" "1"    
}

function create_soft_link_for_ldlinux()
{
    if [ $(id -u) -eq 0 ] ; then
        cmd="if [ ! -e /lib64/ld-linux-aarch64.so.1 ] && [ -e /lib/ld-linux-aarch64.so.1 ]; then \
                ln -s /lib/ld-linux-aarch64.so.1 /lib64/ld-linux-aarch64.so.1 >/dev/null 2>&1; \
             fi"
    else
        cmd="sudo ${INSTALL_SUDO_PATH} create_soft_link_for_ldlinux"
    fi

    exec_cmd_r "$ipaddr" "600" "${cmd}" "1"    
}

function execute_add_libssl_libcrypto()
{
    local boost_version="$1"
    local ipaddr="$2"
    local cmd=""
    if [ $(id -u) -eq 0 ] ; then
        cmd="dst_version=$boost_version; files=\`rpm -qa | grep -E \"*boost[_-](system*|atomic-m*)\" | xargs rpm -ql | grep lib64\`; \
            for file in \$files; \
            do \
                dst_file=\`echo \$file | sed \"s/[0-9.]\+\$/.\$dst_version/g\"\`;  ln -s \$file \$dst_file >/dev/null 2>&1;\
                dst_mt_file=\`echo \$dst_file | sed 's/-mt//g' | sed 's/.so/-mt.so/g'\`; ln -s \$file \$dst_mt_file >/dev/null 2>&1; \
                dst_no_mt_file=\`echo \$dst_file | sed 's/-mt//g'\`; ln -s \$file \$dst_no_mt_file >/dev/null 2>&1;echo -n;\
            done"
    else
        cmd="sudo ${INSTALL_SUDO_PATH} create_soft_link_for_libboost $boost_version"
    fi
    exec_cmd_r "$ipaddr" "600" "${cmd}" "1"
}