#!/bin/bash

######################################################################
#   DESCRIPTION: 服务控制，启动/停止指定的服务
######################################################################
function service_ctl()
{
    local service_name=$1
    local op=$2
    
    service_script="/etc/init.d/${service_name}"
    
    #是否是systemd
    which systemctl > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        systemctl | grep -q "^[ ]*${service_name}.service "
        if [ $? -ne 0 ]; then
            #没有这个服务
            return 0;
        fi
        #使用systemd开关
        if [ "X$op" == "Xstart" ]; then
            systemctl -q start "${service_name}"
            if [ $? -ne 0 ]; then
                echo "[$(date)] ERROR : start service ${service} failed."
                return 1
            fi
            
            systemctl -q enable "${service_name}"
            if [ $? -ne 0 ]; then
                echo "[$(date)] ERROR : start service ${service} failed."
                return 1
            fi
        elif [ "X$op" == "Xstop" ]; then
            systemctl -q stop "${service_name}"
            if [ $? -ne 0 ]; then
                echo "[$(date)] ERROR : stop service ${service} failed."
                return 1
            fi
            
            systemctl -q disable "${service_name}"
            if [ $? -ne 0 ]; then
                echo "[$(date)] ERROR : stop service ${service} failed."
                return 1
            fi
        fi
        
        echo "[$(date)] DEBUG : $op service ${service} successfully."
        return 0
    fi
    
    #检查服务是否存在（注意，有些服务不在/etc/ini.d/目录内），不存在直接返回
    if [ ! -e "${service_script}" ]; then
        echo "[$(date)] DEBUG : Skip to turn off the ${service},because of it does not exist."
        return 0
    fi
    
    #启动操作
    if [ "X$op" == "Xstart" ]; then
        "${service_script}" start > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            echo "[$(date)] ERROR : start service ${service} failed."
            return 1
        fi
    
        chkconfig "${service}" on > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            echo "[$(date)] ERROR : chkconfig on service ${service} failed."
            return 1
        fi
        
        echo "[$(date)] DEBUG : start service ${service} successfully."
    elif [ "X$op" == "Xstop" ]; then
        "${service_script}" stop > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            echo "[$(date)] ERROR : stop service ${service} failed."
            return 1
        fi
    
        chkconfig "${service}" off > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            echo "[$(date)] ERROR : chkconfig off service ${service} failed."
            return 1
        fi
        
        echo "[$(date)] DEBUG : stop service ${service} successfully."
    fi
    
    return 0
}


######################################################################
#   DESCRIPTION: 配置cpuspeed服务
######################################################################
function set_cpuspeed_service()
{
    local name=$1
    local service="cpuspeed"
    local service_script=""
    
    #redhat/centos/oracle的cpuseed以服务的形式存在
    echo "${name}" | grep -E "redhat|centos|oracle" > /dev/null
    if [ $? -eq 0 ]; then
        #检查cpuspeed服务是否存在，不存在直接返回
        if [ ! -e "/etc/init.d/${service}" ]; then
            echo "[$(date)] DEBUG : skip to set the ${service}, because of it does not exist."
            return 0
        fi

        #修改cpuspeeed配置文件
        local CONFIG_FILE="/etc/sysconfig/cpuspeed"
        if [ ! -f $CONFIG_FILE ]; then
            echo "[$(date)] ERROR : $CONFIG_FILE doesn't exist."
            return 1 
        else   
            sed -i 's/^GOVERNOR=.*$/GOVERNOR=performance/g' $CONFIG_FILE
            if [ $? -ne 0 ]; then
                echo "[$(date)] ERROR : failed to modify $CONFIG_FILE."
                return 1
            fi
        fi
        
        chkconfig "${service}" on > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            echo "[$(date)] ERROR : chkconfig on service ${service} failed."
            return 1
        fi
        
        echo "[$(date)] DEBUG : set service ${service} successfully."        
        return 0
    fi
    
    #suse版本需要另外的处理
    echo "${name}" | grep -E "suse" > /dev/null
    if [ $? -eq 0 ]; then
        #只有存在powersave命令才需要进行设置
        which powersave > /dev/null 2>&1
        if [ $? -eq 0 ]; then
            powersave -c | grep "CPU Frequency scaling not supported" > /dev/null
            if [ $? -ne 0 ]; then
                #将cpufreq的模式设置为performance模式，提高性能
                powersave -f > /dev/null 2>&1
                if [ $? -ne 0 ]; then
                    echo "[$(date)] ERROR : set cpufreq to performance mode failed."
                    return 1
                fi
            
                echo "[$(date)] DEBUG : set cpufreq to performance mode successfully."
            fi
        fi
        
        return 0
    fi
    
    return 0
}

######################################################################
#   DESCRIPTION: 关闭irqbalance服务
######################################################################
function stop_irqbalance_service()
{
    local name=$1
    local service="irqbalance"
    local service_script=""
    
    echo "${name}" | grep -E "suse" > /dev/null && ! which systemd  > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        service="irq_balancer"
    fi
    
    service_ctl "${service}" stop
    if [ $? -ne 0 ]; then
        echo "[$(date)] ERROR : service_ctl ${service} stop failed."
        return 1
    fi
    
    return 0
}

######################################################################
#   DESCRIPTION: 关闭操作系统中可能会影响性能的服务
######################################################################
function main()
{
    local os_name=""
    local base_dir=$(dirname $0)
    local adapter_script="${base_dir}/adapter.sh"
    local osperf_conf="${base_dir}/osperf.conf"
    
    . "${osperf_conf}"
    source "${adapter_script}"
    os_name=$(get_osname)
    if [ $? -ne 0 ]; then
        echo "[$(date)] ERROR : Don't support [$(cat /proc/version)]"
        return 1
    fi
    
    if [ "${osperf_service_stop_cpuspeed}" -eq 1 ]; then
        #关闭cpuspeed服务
        set_cpuspeed_service "${os_name}"
        if [ $? -ne 0 ]; then
            echo "[$(date)] ERROR : Exception occurs when turning off the cpuspeed service."
            return 1
        fi
    fi
    
    if [ "${osperf_service_stop_irqbalance}" -eq 1 ]; then
        #关闭irqbalance服务
        stop_irqbalance_service "${os_name}"
        if [ $? -ne 0 ]; then
            echo "[$(date)] ERROR : Exception occurs when turning off the irqbalance service."
            return 1
        fi
    fi
    
    echo "[$(date)] INFO : Turning off the service successfully."
    return 0
}

main
exit $?


