#!/bin/bash

#连接kms服务器的配置信息
kms_conf_file="/usr/local/kmsLuksMgt/ini/kms_info"

#基于kms服务加密的分区挂载信息
kms_mount_info="/usr/local/kmsLuksMgt/ini/kms_encrypt_devs"

#kmc加密工具
kmc_tool_dir="/usr/local/kmsLuksMgt/kmc/"

#秘钥材料
material=""

#日志文件
LOGPATH="/var/log/kmsLuksMgt/"
LOGFILE="/var/log/kmsLuksMgt/kmsLuksMgt.log"


if [ -f "$HOME/.bash_profile" ]; then
    . $HOME/.bash_profile  
fi
if [[ $UID -ne 0 && -n "${INSTALL_SUDO_PATH}" ]]; then
    #非root用户，使用sudo
    CMD_FINDFS="sudo ${INSTALL_SUDO_PATH} preinstall_findfs"
    CMD_MOUNT="sudo ${INSTALL_SUDO_PATH} preinstall_mount"
    CMD_CAT="sudo ${INSTALL_SUDO_PATH} preinstall_cat"
    CMD_CRYPTSETUP="sudo ${INSTALL_SUDO_PATH} preinstall_cryptsetup"
    CMD_MOUNT_COMMAND="sudo /bin/mount"
else
    CMD_FINDFS="findfs"
    CMD_MOUNT="preinstall_mount"
    CMD_CAT="cat"
    CMD_CRYPTSETUP="cryptsetup"
    CMD_MOUNT_COMMAND="/bin/mount"
fi

########################################################################
# 函数描述： 记录日志封装函数
# 参数1说明：0 只显示终端
#            1 只记录日志文件
#            2 显示终端，并记录日志文件（一般用于主要输出信息）
# 参数2说明：日志内容
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function logme()
{
    local log_mode=$1
    local log_info="$2"
    
    
    if [ $# -ne 2 ]; then
        echo "logme para invalid."
        return 1
    fi
    
    #root安装，所以没有路径要实时创建
    mkdir -p "${LOGPATH}"
    if [ $? -ne 0 ]; then
        echo "[$(date)] [EMER] diskmgtmodule ins: mkdir -p ${LOGPATH} failed."
        echo "[$(date)] ${log_info}"
        return 1
    fi
    
    chmod 700 "${LOGPATH}"
    
    if [ ${log_mode} -eq 0 ]; then
        echo "[$(date)][${IPADDR}] ${log_info}"
    elif [ ${log_mode} -eq 1 ]; then
        echo "[$(date)][${IPADDR}] ${log_info}" >>"${LOGFILE}"
    elif [ ${log_mode} -eq 2 ]; then
        echo "[$(date)][${IPADDR}] ${log_info}" | tee -a "${LOGFILE}"  #通过tee追加显示
    else
        echo "logme invalid mode para."
        return 1
    fi
    
    return 0
}

########################################################################
# 函数描述： mount指定的磁盘
# 参数1说明：设备名
#
# 返回值  ：0 成功
#           1 失败
########################################################################
function preinstall_mount()
{
    if [ -z "$1" ]; then
        echo "arguments is null."
        return 1
    fi
    
    if [ ! -e "$1" ]; then
        echo "dev $1 does not exist."
        return 1
    fi
    
    mount "$1" "$2" -o noatime,nodiratime
    return $?
}


########################################################################
# 函数描述： 创建加密磁盘映射
# 参数1说明：磁盘信息
#            
# 参数2说明: 映射目录
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function mapper_partation()
{

    local partation="$1"
    local dev_mapper="$2"
    
    kms_server=$(cat ${kms_conf_file} | grep "^[[:space:]]*kms_server=" | awk '{print $1}' | sed 's/kms_server=//g' | head -n 1 2>/dev/null)
    if [ -z "$kms_server" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get kms server from $kms_conf_file."
        return 1
    fi
    
    kms_key_version_name=$(cat ${kms_conf_file} | grep "^[[:space:]]*kms_key_version_name=" | awk '{print $1}' | sed 's/kms_key_version_name=//g' | head -n 1 2>/dev/null)
    if [ -z "$kms_key_version_name" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get kms key version name from $kms_conf_file."
        return 1
    fi

    #根据密钥版本获取秘钥材料
    cd $kmc_tool_dir > /dev/null 2>&1
    material=$(./kmc_tool -g -i "GET https://${kms_server}/kms/v1/keyversion/${kms_key_version_name}")
    cd - > /dev/null 2>&1
    if [ -z "${material}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to get Material from ${kms_server}."
        return 1
    fi
    
    #为避免加密分区被占用，先close，失败后也close
    ${CMD_CRYPTSETUP} luksClose "${dev_mapper}" >/dev/null
    
    #建立加密磁盘映射
    echo "[INFO] [${FUNCNAME}:${LINENO}] Start mapper encrypted $partation to $dev_mapper."
expect <<EOF
set timeout 30;
spawn -noecho ${CMD_CRYPTSETUP} luksOpen "$partation" "$dev_mapper" >> "$LOGFILE" 2>&1
expect   "passphrase"
send "${material}\r"
expect eof
EOF
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to mapper $partation to $dev_mapper."
        ${CMD_CRYPTSETUP} luksClose "${dev_mapper}" >/dev/null
        return 1
    fi
    
    
    if [ ! -b "/dev/mapper/$dev_mapper" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] devpart /dev/mapper/$dev_mapper is not block."
        return 1
    fi
    
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] convert $partation to encrypted $dev_mapper. End encrypt $partation."
    return 0
}


#根据挂载信息进行目录挂载
function mount_partation()
{
    #读取配置并挂载
    while read line
    do 
        echo ${line} | grep -e "^UUID="
        if [ $? -ne 0 ]; then
            continue
        fi
        
        uuid=$(echo ${line} | awk '{print $1}')
        if [ -z ${uuid} ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to get uuid from ${mount_info}."
            return 1
        fi
        
        partation=$(${CMD_FINDFS} ${uuid})
        if [ -z ${partation} ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to get partation from uuid=${uuid}."
            return 1
        fi
        
        dev_mapper=$(echo ${line} | awk '{print $2}')
        if [ -z ${dev_mapper} ]; then
            logme 2  "[ERROR] [${FUNCNAME}:${LINENO}] failed to get dev_mapper from ${mount_info}."
            return 1
        fi

        dir=$(echo ${line} | awk '{print $3}')
        if [ -z ${dev_mapper} ]; then
            logme 2  "[ERROR] [${FUNCNAME}:${LINENO}] failed to get dir from ${mount_info}."
            return 1
        fi
        
        logme 2  "[INFO] [${FUNCNAME}:${LINENO}] Start to mount encrypt partation."
        ${CMD_MOUNT_COMMAND} | grep ${dev_mapper}
        if [ $? -eq 0 ]; then
            logme 2  "[INFO] [${FUNCNAME}:${LINENO}] ${dev_mapper} has mounted"
            continue
        fi
        
        mapper_partation ${partation} ${dev_mapper}
        if [ $? -ne 0 ]; then
            logme 2  "[ERROR] [${FUNCNAME}:${LINENO}] mapper failed."
        fi
        
        
        #避免红帽systemd-219-19.el7版本的bug(在某些场景下会将已经mount的分区自动umount掉)
        udevadm settle  >/dev/null 2>&1
        #规避分区挂载上之后马上又被自动umount掉（OS bug）
        systemctl daemon-reload >/dev/null 2>&1
        dev_mapper=/dev/mapper/$dev_mapper
        ${CMD_MOUNT} ${dev_mapper} ${dir}
        if [ $? -ne 0 ]; then
            logme 2  "[ERROR] [${FUNCNAME}:${LINENO}] mount ${dir} to ${dev_mapper} failed."
        fi
    done < ${kms_mount_info}
}

function main(){
    #挂载加密目录
    mount_partation || return 1
}

main



