#!/bin/bash
##############################################################################################
## Function Description:
#### This script is used to expand disk partitions in the cloud environment.
## Constraints:
#### 1. Only the root user can run this script.
#### 2. The disk partition format is GPT.
#### 3. A single partition occupies the entire disk space (one disk occupies one partition).
#### 3. Only partitions of data disks can be expanded. System disks are not supported.
#### 4. The disk partition file system is ext.
##############################################################################################

declare DISK_MGT_SERVICE_PATH="/usr/local/diskmgt"
declare LOGPATH="/var/log/diskmgt"
declare LOGFILE="${LOGPATH}/expand_disk_partition.log"
declare g_diskSizeExpandThreshold=100  # unit: MB
declare g_cloudFlagFile="${DISK_MGT_SERVICE_PATH}/ini/cloud" # Flag file, which is used to mark whether the environment is on the cloud.
# content: alarm-action#mount-point disk-path#disk-size#dev-part-path#dev-part-size#dev-part-serial-num#dev-fs-size
declare g_sendFailedAlarmRecord="${DISK_MGT_SERVICE_PATH}/inner/.expand_dev_failed_alarm_record.lst"
# content: mount-point disk-path#disk-size#dev-part-path#dev-part-size#dev-part-serial-num#dev-fs-size
declare g_reportAlarmRecord="${DISK_MGT_SERVICE_PATH}/inner/.expand_dev_report_alarm_record.lst"
declare g_alarmReportAction="REPORT_ALARM"
declare g_alarmClearAction="CLEAR_ALARM"
declare g_updateFailedAlarmRecord="update_failed_alarm_record"
declare g_deleteFailedAlarmRecord="delete_failed_alarm_record"
declare g_updateAlarmRecord="update_alarm_record"
declare g_deleteAlarmRecord="delete_alarm_record"
declare g_mountPoint="$1"
declare g_ommUserName="omm"
declare g_alarmTool=""
declare g_expandDevAlarmID="12187"
declare g_expandDevAlarmLevel="3"

function log() {
    local log_mode=$1
    local log_info="$2"
    local script_log="${DISK_MGT_SERVICE_PATH}/inner/scriptlog.sh"

    if [ $# -ne 2 ]; then
        echo "log para invalid."
        return 1
    fi

    local dateStr="$(date -d today +'%Y-%m-%d %H:%M:%S.%3N')"

    if [ ${log_mode} -eq 0 ]; then
        echo "[${dateStr}] ${log_info}"
    elif [ ${log_mode} -eq 1 ]; then
        echo "[${dateStr}] [$$] ${log_info}" >>"${LOGFILE}"
    elif [ ${log_mode} -eq 2 ]; then
        echo "[${dateStr}] [$$] ${log_info}" | tee -a "${LOGFILE}" #通过tee追加显示
    else
        echo "log invalid mode parameter."
        return 1
    fi

    if [ -f "${LOGFILE}" ]; then
        chmod 640 "${LOGFILE}"
        "${script_log}" "${LOGFILE}"
    fi

    return 0
}

function transferByteToMB() {
    local byteVal=$1
    echo "$[byteVal/1024/1024]"
}

function build_alarm_info() {
    local partitionInfo="$(lsblk -o KNAME,PATH,PKNAME,SIZE,MOUNTPOINT | grep -w ${g_mountPoint})"
    local devName="$(echo ${partitionInfo} | awk -F '[ ]+' '{print $1}')"
    local devPath="$(echo ${partitionInfo} | awk -F '[ ]+' '{print $2}')"
    local pDevName="$(echo ${partitionInfo} | awk -F '[ ]+' '{print $3}')"
    local devSize="$(echo ${partitionInfo} | awk -F '[ ]+' '{print $4}')"
    local diskInfo="$(lsblk -b -o KNAME,PATH,SIZE | grep -w ${diskPath})"
    local diskSize="$(lsblk -o KNAME,PATH,SIZE | grep -w /dev/${pDevName} | awk -F '[ ]+' '{print $3}')"  # unit: B
    local devSerialNum="$(ls -l /dev/disk/by-id | grep -w "${devName}" | awk 'BEGIN{ORS=","} {print $9}')"
    devSerialNum="${devSerialNum%,*}"
    local devFsSize="$(df -h | grep -w ${g_mountPoint} | awk -F '[ ]+' '{print $2}')"
    echo "/dev/${pDevName}#${diskSize}#${devPath}#${devSize}#${devSerialNum}#${devFsSize}"
}

function get_send_failed_alarm_info() {
    local alarmAction="$1"
    local recordKey="${alarmAction}#${g_mountPoint}"
    local record="$(cat ${g_sendFailedAlarmRecord} | grep -w ${recordKey})"
    local alarmInfo="$(echo ${record} | awk -F '[ ]+' '{print $2}')"
    echo "${alarmInfo}"
}

function update_send_failed_alarm_record() {
    local updateType="$1"
    local alarmAction="$2"
    local sendFailedAlarmRecordTmp="${g_sendFailedAlarmRecord}.tmp"
    local recordKey="${alarmAction}#${g_mountPoint}"
    local recordInfo="${recordKey} $3" # recordKey alarmInfo
    cat "${g_sendFailedAlarmRecord}" | grep -vw "${recordKey}" > "${sendFailedAlarmRecordTmp}"
    cat "${sendFailedAlarmRecordTmp}" > "${g_sendFailedAlarmRecord}"
    rm -rf "${sendFailedAlarmRecordTmp}"
    if [ "${updateType}" != "${g_deleteFailedAlarmRecord}" ]; then
        echo "${recordInfo}" >> "${g_sendFailedAlarmRecord}"
        log 1 "[INFO] [${FUNCNAME}:${LINENO}] save send failed alarm record(${recordInfo})"
    fi
}

function update_report_alarm_record() {
    local updateType="$1"
    local reportAlarmRecordTmp="${g_reportAlarmRecord}.tmp"
    local recordKey="${g_mountPoint}"
    local recordInfo="${recordKey} $2"  # recordKey alarmInfo
    cat "${g_reportAlarmRecord}" | grep -vw "${recordKey}" > "${reportAlarmRecordTmp}"
    cat "${reportAlarmRecordTmp}" > "${g_reportAlarmRecord}"
    rm -rf "${reportAlarmRecordTmp}"
    if [ "${updateType}" != "${g_deleteAlarmRecord}" ]; then
        echo "${recordInfo}" >> "${g_reportAlarmRecord}"
        log 1 "[INFO] [${FUNCNAME}:${LINENO}] save report alarm record(${recordInfo})"
    fi
}

function get_report_alarm_info() {
    local recordKey="${g_mountPoint}"
    local record="$(cat ${g_reportAlarmRecord} | grep -w ${recordKey})"
    local alarmInfo="$(echo ${record} | awk -F '[ ]+' '{print $2}')"
    echo "${alarmInfo}"
}

function send_alarm() {
    local alarmCategory="1" # default: clear alarm
    local alarmRecordAction="${g_deleteAlarmRecord}"
    local alarmAction="$1"
    local alarmInfo="$2"
    if [ "${alarmAction}" == "${g_alarmReportAction}" ]; then
        alarmCategory="0"
        alarmRecordAction="${g_updateAlarmRecord}"
    fi

    local devSerialNum="$(echo ${alarmInfo} | awk -F '#' '{print $5}')"
    local alarmCause=0
    local occurTime=""
    local mocName="${HOSTNAME}"
    local resourceID="diskmgtd"
    local locationInfo="Manager;NA;${HOSTNAME};${g_mountPoint}"
    local additionalInfo="${devSerialNum}"

    log 1 "[INFO] [${FUNCNAME}:${LINENO}] ${alarmAction}: mount-point[${g_mountPoint}], alarm-category[${alarmCategory}], alarm-info[${alarmInfo}])"

    su - "${g_ommUserName}" -c "\"${g_alarmTool}\" \"${g_expandDevAlarmID}\" \"${alarmCategory}\" \"${g_expandDevAlarmLevel}\" \"${alarmCause}\" \"${occurTime}\" \"${mocName}\" \"${resourceID}\" \"${locationInfo}\" \"${additionalInfo}\""
    local ret=$?
    local sendAlarmInfo="${g_alarmTool} 1-ID[${g_expandDevAlarmID}] 2-Category[${alarmCategory}] 3-Level-[${g_expandDevAlarmLevel}] 4-Cause[${alarmCause}] 5-Time[${occurTime}] 6-Moc[${mocName}] 7-Res[${resourceID}] 8-LocalInfo[${locationInfo}] 9-AddInfo[${additionalInfo}]"
    if [ "${ret}" -ne 0 ]; then
        update_send_failed_alarm_record "${g_updateFailedAlarmRecord}" "${alarmAction}" "${alarmInfo}"
        log 1 "[WARN] [${FUNCNAME}:${LINENO}] Failed to send alarm(ret=${ret}): ${sendAlarmInfo}"
    else
        update_send_failed_alarm_record "${g_deleteFailedAlarmRecord}" "${alarmAction}" "${alarmInfo}"
        update_report_alarm_record "${alarmRecordAction}" "${alarmInfo}"
        log 1 "[INFO] [${FUNCNAME}:${LINENO}] Success to send alarm: ${sendAlarmInfo}"
    fi
}

function resend_failure_alarm() {
    log 1 "[INFO] [${FUNCNAME}:${LINENO}] Start to resend failure alarms"

    # send report alarm
    local alarmInfo="$(get_send_failed_alarm_info ${g_alarmReportAction})"
    if [ ! -z "${alarmInfo}" ]; then
        send_alarm "${g_alarmReportAction}" "${alarmInfo}"
    fi

    # send clear alarm
    alarmInfo="$(get_send_failed_alarm_info ${g_alarmClearAction})"
    if [ ! -z "${alarmInfo}" ]; then
        send_alarm "${g_alarmClearAction}" "${alarmInfo}"
    fi

    log 1 "[INFO] [${FUNCNAME}:${LINENO}] End to resend failure alarms"
}

function do_expand_disk_part_fs() {
    local devPath="$1"
    log 1 "[INFO] [${FUNCNAME}:${LINENO}] Start to expand disk partition file system size(dev-path:[${devPath}])"
    local preDevPartFsSize="$(df -h | grep -w ${devPath} | awk -F '[ ]+' '{print $2}')"
    local expandDiskPartitionFSCmd="resize2fs ${devPath}"
    log 1 "[INFO] [${FUNCNAME}:${LINENO}] Execute expand-disk-partition-file-system command:${expandDiskPartitionFSCmd}"
    ${expandDiskPartitionFSCmd} >> "${LOGFILE}" 2>&1
    if [ $? -ne 0 ]; then
        send_alarm "${g_alarmReportAction}" "$(build_alarm_info)"
        log 1 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to execute expand-disk-partition-file-system command:${expandDiskPartitionFSCmd}"
        return 1
    fi

    local currentDevPartFsSize="$(df -h | grep -w ${devPath} | awk -F '[ ]+' '{print $2}')"
    send_alarm "${g_alarmClearAction}" "$(build_alarm_info)"
    log 1 "[INFO] [${FUNCNAME}:${LINENO}] The file system size of the disk partition[${devPath}] has changed[${preDevPartFsSize}->${currentDevPartFsSize}]."
    log 1 "[INFO] [${FUNCNAME}:${LINENO}] End to expand disk partition file system size(dev-path:[${devPath}])"
}

function do_expand_disk_partition() {
    local diskPath="$1"
    local devPath="$2"
    log 1 "[INFO] [${FUNCNAME}:${LINENO}] Start to expand disk partition(dev-path:[${devPath}], disk-path:[${diskPath}])"

    which growpart > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        send_alarm "${g_alarmReportAction}" "$(build_alarm_info)"
        log 1 "[ERROR] [${FUNCNAME}:${LINENO}] The growpart expansion tool does not exist."
        return 1
    fi

    local preDevPartSize="$(lsblk -o KNAME,PATH,SIZE | grep -w ${devPath} | awk -F '[ ]+' '{print $3}')"
    local devNumber="${devPath#*${diskPath}}"
    local expandDiskPartitionSizeCmd="growpart ${diskPath} ${devNumber}"
    log 1 "[INFO] [${FUNCNAME}:${LINENO}] Execute expand-disk-partition-size command:${expandDiskPartitionSizeCmd}"
    ${expandDiskPartitionSizeCmd} >> "${LOGFILE}" 2>&1
    if [ $? -ne 0 ]; then
        send_alarm "${g_alarmReportAction}" "$(build_alarm_info)"
        log 1 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to execute expand-disk-partition-size command:${expandDiskPartitionSizeCmd}"
        return 1
    fi

    local currentDevPartSize="$(lsblk -o KNAME,PATH,SIZE | grep -w ${devPath} | awk -F '[ ]+' '{print $3}')"
    log 1 "[INFO] [${FUNCNAME}:${LINENO}] The disk partition[${devPath}] space has changed[${preDevPartSize}->${currentDevPartSize}]."

    log 1 "[INFO] [${FUNCNAME}:${LINENO}] End to expand disk partition(dev-path:[${devPath}], disk-path:[${diskPath}])."
}

function expand_disk_partition() {
    # obtain partition info
    local partitionInfo="$(lsblk -b -o KNAME,PATH,PKNAME,SIZE,MOUNTPOINT | grep -w ${g_mountPoint})"
    local devPath="$(echo ${partitionInfo} | awk -F '[ ]+' '{print $2}')"
    local pDevName="$(echo ${partitionInfo} | awk -F '[ ]+' '{print $3}')"
    local devSize="$(echo ${partitionInfo} | awk -F '[ ]+' '{print $4}')" # unit: B
    devSize="$(transferByteToMB ${devSize})"

    # obtain disk info
    local diskPath="/dev/${pDevName}"
    local diskInfo="$(lsblk -b -o KNAME,PATH,SIZE | grep -w ${diskPath})"
    local diskSize="$(echo ${diskInfo} | awk -F '[ ]+' '{print $3}')"  # unit: B
    diskSize="$(transferByteToMB ${diskSize})"
    log 1 "[INFO] [${FUNCNAME}:${LINENO}] mount-point:[${g_mountPoint}], dev-path:[${devPath}], dev-size:[${devSize}MB], disk-path:[${diskPath}], disk-size:[${diskSize}MB]"

    local expandDiskSize=`expr ${diskSize} - ${devSize}`
    if [ ${expandDiskSize} -gt ${g_diskSizeExpandThreshold} ]; then
        do_expand_disk_partition "${diskPath}" "${devPath}"
        if [ $? -ne 0 ]; then
            return 1
        fi
        do_expand_disk_part_fs "${devPath}"
    else
        # Handling scenario: The partition expansion is complete but the alarm is not cleared. Then, the alarm is cleared.
        local reportAlarmInfo="$(get_report_alarm_info)"
        if [ ! -z "${reportAlarmInfo}" ]; then
            do_expand_disk_part_fs "${devPath}"
            if [ $? -ne 0 ]; then
                return 1
            else
                log 1 "[INFO] [${FUNCNAME}:${LINENO}] Clear the alarm[${reportAlarmInfo}] that has been sent but is not cleared."
            fi
        fi
    fi
}

function init_alarm_tool()
{
    local nodeAgentHome="$(su - ${g_ommUserName} -c 'echo $NODE_AGENT_HOME')"
    local supressionAlarmScript="${nodeAgentHome}/bin/supressionSendAlarm.sh"
    local alarmScript="${nodeAgentHome}/bin/sendAlarm"

    if [ -f "${supressionAlarmScript}" ]; then
        g_alarmTool="${supressionAlarmScript}"
        return 0
    fi

    if [ -f "${alarmScript}" ]; then
        g_alarmTool="${alarmScript}"
        return 0
    fi

    log 1 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to init alarm tool(NODE_AGENT_HOME:[$NODE_AGENT_HOME])!"
    return 1
}

function init() {
    if [ ! -d "${LOGPATH}" ]; then
        mkdir -p "${LOGPATH}"
    fi

    chmod 700 "${LOGPATH}"

    if [ ! -f "${g_sendFailedAlarmRecord}" ]; then
        touch "${g_sendFailedAlarmRecord}"
    fi

    if [ ! -f "${g_reportAlarmRecord}" ]; then
        touch "${g_reportAlarmRecord}"
    fi

    init_alarm_tool
    return $?
}

function main() {
    if [[ $UID -ne 0 ]]; then
        log 1 "[ERROR] [${FUNCNAME}:${LINENO}] Only the root user can run this script."
        exit 1
    fi

    # The cloud environment is not directly exited.
    if [ ! -f "${g_cloudFlagFile}" ]; then
        exit 1
    fi

    init
    if [ $? -ne 0 ]; then
        exit 1
    fi

    resend_failure_alarm
    expand_disk_partition
}

main "$@"
exit $?