#!/bin/bash

declare CRYPT_TAB_INFO="/etc/crypttab"
declare FSTAB_INFO="/etc/fstab"
declare DISK_MGT_SERVICE_PATH="/usr/local/diskmgt"
declare ENCRYPT_PARTITION_INFO="${DISK_MGT_SERVICE_PATH}/ini-plugin/keys/encrypt_partition_info.lst"
declare SCC_CONF="${DISK_MGT_SERVICE_PATH}/scc/config/scc.conf"
declare DECRYPT_CMD="/usr/local/seccomponent/bin/CryptoAPI -d "

#日志文件
declare LOGPATH="/var/log/diskmgt"
declare LOGFILE="${LOGPATH}/mount_encrypt_partition.log"

if [ -f "$HOME/.bash_profile" ]; then
    . $HOME/.bash_profile
fi
if [[ $UID -ne 0 && -n "${INSTALL_SUDO_PATH}" ]]; then
    #非root用户，使用sudo
    CMD_FINDFS="sudo ${INSTALL_SUDO_PATH} preinstall_findfs"
    CMD_MOUNT="sudo ${INSTALL_SUDO_PATH} preinstall_mount"
    CMD_CAT="sudo ${INSTALL_SUDO_PATH} preinstall_cat"
    CMD_CRYPTSETUP="sudo ${INSTALL_SUDO_PATH} preinstall_cryptsetup"
    CMD_MOUNT_COMMAND="sudo /bin/mount"
else
    CMD_FINDFS="findfs"
    CMD_MOUNT="preinstall_mount"
    CMD_CAT="cat"
    CMD_CRYPTSETUP="cryptsetup"
    CMD_MOUNT_COMMAND="/bin/mount"
fi

########################################################################
# 函数描述： 记录日志封装函数
# 参数1说明：0 只显示终端
#            1 只记录日志文件
#            2 显示终端，并记录日志文件（一般用于主要输出信息）
# 参数2说明：日志内容
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function log() {
    local log_mode=$1
    local log_info="$2"
    local script_log="${DISK_MGT_SERVICE_PATH}/inner/scriptlog.sh"

    if [ $# -ne 2 ]; then
        echo "log para invalid."
        return 1
    fi

    #root安装，所以没有路径要实时创建
    mkdir -p "${LOGPATH}"
    if [ $? -ne 0 ]; then
        echo "[$(date)] [ERROR] diskmgt-module ins: mkdir -p ${LOGPATH} failed."
        echo "[$(date)] ${log_info}"
        return 1
    fi

    chmod 700 "${LOGPATH}"

    if [ ${log_mode} -eq 0 ]; then
        echo "[$(date -d today +'%Y-%m-%d %H:%M:%S')] ${log_info}"
    elif [ ${log_mode} -eq 1 ]; then
        echo "[$(date -d today +'%Y-%m-%d %H:%M:%S')] [$$] ${log_info}" >>"${LOGFILE}"
    elif [ ${log_mode} -eq 2 ]; then
        echo "[$(date -d today +'%Y-%m-%d %H:%M:%S')] [$$] ${log_info}" | tee -a "${LOGFILE}" #通过tee追加显示
    else
        echo "log invalid mode parameter."
        return 1
    fi

    if [ -f "${LOGFILE}" ]; then
        chmod 640 "${LOGFILE}"
        "${script_log}" "${LOGFILE}"
    fi

    return 0
}

function get_decrypt_word() {
    local cipher="$1"
    if [ -z "${cipher}" ] || [ ! -f "${SCC_CONF}" ]; then
        log 1 "[ERROR] [${FUNCNAME}:${LINENO}] The cipher is empty or the scc-conf-file is not exist."
        return 1
    fi

    export "SCC_CONF=${SCC_CONF}"
    local passwd="$(echo "${cipher}" | ${DECRYPT_CMD} -d)"
    local retVal="$?"
    passwd=$(echo "${passwd}" | awk -F ':' '{print $2}')
    if [ $retVal -ne 0 ] || [ -z "${passwd}" ]; then
        echo ""
        return 1
    fi

    echo "${passwd}"
    return 0
}

########################################################################
# 函数描述： mount指定的磁盘
# 参数1说明：设备名
#
# 返回值  ：0 成功
#           1 失败
########################################################################
function preinstall_mount() {
    if [ -z "$1" ]; then
        echo "arguments is null."
        return 1
    fi

    if [ ! -e "$1" ]; then
        echo "dev $1 does not exist."
        return 1
    fi
    mount "$1" "$2" -o defaults,noatime,nodiratime,nofail
    return $?
}

########################################################################
# 函数描述： 创建加密磁盘映射
# 参数1说明：磁盘信息
#
# 参数2说明: 映射目录
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function mapper_partition() {

    local partition="$1"
    local dev_mapper="$2"
    local key_file="$3"

    #为避免加密分区被占用，先close，失败后也close
    ${CMD_CRYPTSETUP} luksClose "${dev_mapper}" >/dev/null

    #建立加密磁盘映射
    log 2 "[INFO] [${FUNCNAME}:${LINENO}] Start mapper encrypted $partition to $dev_mapper."
    ${CMD_CRYPTSETUP} luksOpen "${partition}" "${dev_mapper}" -d "${key_file}"
    if [ $? -ne 0 ]; then
        log 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to mapper $partition to $dev_mapper."
        ${CMD_CRYPTSETUP} luksClose "${dev_mapper}" >/dev/null
        return 1
    fi

    if [ ! -b "/dev/mapper/$dev_mapper" ]; then
        log 2 "[ERROR] [${FUNCNAME}:${LINENO}] The dev-part[/dev/mapper/$dev_mapper] is not block."
        return 1
    fi

    log 2 "[INFO] [${FUNCNAME}:${LINENO}] convert $partition to encrypted $dev_mapper. End encrypt $partition."
    return 0
}

#根据挂载信息进行目录挂载
function mount_partition() {
    local dev_partition_info="$1"
    local dev_mapper=$(echo "${dev_partition_info}" | awk -F '[ ]+' '{print $1}')
    if [ -z ${dev_mapper} ]; then
        log 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to get dev_mapper from ${CRYPT_TAB_INFO}."
        return 1
    fi

    ${CMD_MOUNT_COMMAND} | grep -w "${dev_mapper}"
    if [ $? -eq 0 ]; then
        log 2 "[INFO] [${FUNCNAME}:${LINENO}] ${dev_mapper} has mounted"
        return 0
    fi

    local dev_mapper_uuid=$(echo "${dev_partition_info}" | awk -F '[ ]+' '{print $2}')
    if [ -z ${dev_mapper_uuid} ]; then
        log 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to get dev_mapper_uuid from ${CRYPT_TAB_INFO}."
        return 1
    fi

    local key_file=$(echo "${dev_partition_info}" | awk -F '[ ]+' '{print $3}')
    if [ -z ${key_file} ]; then
        log 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to get key_file from ${CRYPT_TAB_INFO}."
        return 1
    fi

    local partition=$(${CMD_FINDFS} "${dev_mapper_uuid}")
    if [ -z ${partition} ]; then
        log 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to get partition from dev_mapper_uuid=${dev_mapper_uuid}."
        return 1
    fi

    local mount_dir=$(${CMD_CAT} "${ENCRYPT_PARTITION_INFO}" | grep -w "${dev_mapper}" | awk -F '[ ]+' '{print $3}')
    if [ -z ${mount_dir} ]; then
        log 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to get mount_dir from ${ENCRYPT_PARTITION_INFO}."
        return 1
    fi

    local fs_type=$(${CMD_CAT} "${ENCRYPT_PARTITION_INFO}" | grep -w "${dev_mapper}" | awk -F '[ ]+' '{print $4}')
    if [ -z ${fs_type} ]; then
        log 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to get fs_type from ${ENCRYPT_PARTITION_INFO}."
        return 1
    fi

    # fstab中设备的uuid
    local dev_uuid=$(${CMD_CAT} "${ENCRYPT_PARTITION_INFO}" | grep -w "${dev_mapper}" | awk -F '[ ]+' '{print $2}')
    if [ -z ${dev_uuid} ]; then
        log 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to get dev_uuid from ${ENCRYPT_PARTITION_INFO}."
        return 1
    fi

    log 2 "[INFO] [${FUNCNAME}:${LINENO}] Start to mount encrypt partition."
    local passwd="$(${CMD_CAT} ${key_file})"
    passwd="$(get_decrypt_word ${passwd})"
    if [ $? -ne 0 ] || [ -z ${passwd} ]; then
        log 2 "[INFO] [${FUNCNAME}:${LINENO}] failed decrypt password with key_file[${key_file}] from ${CRYPT_TAB_INFO}, so direct use the file[${key_file}]."
        mapper_partition "${partition}" "${dev_mapper}" "${key_file}"
    else
        log 2 "[INFO] [${FUNCNAME}:${LINENO}] Use the decrypted key file."
        local cipherKeyFile="${key_file}.cipher"
        echo "${passwd}" >${cipherKeyFile}
        mapper_partition "${partition}" "${dev_mapper}" "${cipherKeyFile}"
        rm -rf ${cipherKeyFile}
    fi

    if [ $? -ne 0 ]; then
        log 2 "[ERROR] [${FUNCNAME}:${LINENO}] mapper failed."
        return 1
    fi

    #避免红帽systemd-219-19.el7版本的bug(在某些场景下会将已经mount的分区自动umount掉)
    udevadm settle >/dev/null 2>&1
    #规避分区挂载上之后马上又被自动umount掉（OS bug）
    systemctl daemon-reload >/dev/null 2>&1
    dev_mapper="/dev/mapper/$dev_mapper"

    ${CMD_MOUNT_COMMAND} | grep -w "${dev_mapper}" >>"${LOGFILE}" 2>&1
    if [ $? -eq 0 ]; then
        log 2 "[INFO] [${FUNCNAME}:${LINENO}] ${dev_mapper} has mounted"
    else
        log 2 "[INFO] [${FUNCNAME}:${LINENO}] Execute-mount-cmd:${CMD_MOUNT} ${dev_mapper} ${mount_dir}"
        ${CMD_MOUNT} "${dev_mapper}" "${mount_dir}"
        if [ $? -ne 0 ]; then
            log 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount ${mount_dir} to ${dev_mapper} failed."
            return 1
        fi
    fi

    grep "^${dev_uuid}" "${FSTAB_INFO}" >>"${LOGFILE}" 2>&1
    if [ $? -eq 0 ]; then
        log 1 "[INFO] [${FUNCNAME}:${LINENO}] The mount info[${dev_uuid}] is exist in file[${FSTAB_INFO}]"
    else
        log 1 "[INFO] [${FUNCNAME}:${LINENO}] The mount info[${dev_uuid}] is not exist in file[${FSTAB_INFO}], so add mount info."
        echo "${dev_uuid} ${mount_dir} ${fs_type} defaults,noatime,nodiratime,nofail 1 0" >>"${FSTAB_INFO}"
    fi
    log 2 "[INFO] [${FUNCNAME}:${LINENO}] Success to mount ${mount_dir} to ${dev_mapper}."
}

function mount_partitions() {
    #读取配置并挂载
    ${CMD_CAT} "${CRYPT_TAB_INFO}" | sed "/^#/d" | sed "/^$/d" | while read line; do
        echo "${line}" | grep -e "^encrypted"
        if [ $? -ne 0 ]; then
            continue
        fi
        mount_partition "${line}"
        if [ $? -ne 0 ]; then
            log 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to mount partition[${line}]"
        fi
    done
}

function main() {
    #挂载加密目录
    mount_partitions
}

main
