#!/bin/bash

declare DECRYPT_CMD="/usr/local/seccomponent/bin/CryptoAPI -d "
declare ENCRYPT_CMD="/usr/local/seccomponent/bin/CryptoAPI -e "
declare SECCOMPONENT_PACKAGE_NAME="seccomponent-1.1.8-release" # scc安装包名
declare BASE_CHARS="abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVXYZ01223456789~@%^-_=+[].?"
declare BASE_SPEC_CHARS="~@%^-_=+[].?"
declare DIGIT_CHARS="0123456789"
declare ALP_CHARS="abcdefghijklmnopqrstuvwxyz"
declare PWD_LEN=16

declare ERR_CODE_SCC_CONF_FILE_NOT_EXIST=100
declare ERR_CODE_CIPHER_IS_EMPTY=101
declare ERR_CODE_DECRYPT_PWD=102
declare ERR_CODE_ENCRYPT_PWD=103
declare ERR_CODE_INIT_SCC=103

################################################################################
# Function: update_rpm_package
# Description: update seccomponent package
# Parameter:
#   input:
#   $1    package file path
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function update_rpm_package()
{
    local rpmName="$1"
    local rpmPath="$2"

    if [ -z ${rpmName} ]; then
        echo "RPM name ${rpmName} is empty."
        return 1
    fi

    if [ -z ${rpmPath} ]; then
        echo "RPM file ${rpmPath} does not exist."
        return 1
    fi

    rpm -q ${rpmName}
    if [ $? -eq 0 ]; then
        echo "RPM ${rpmName} is already installed."
        return 0
    fi

    rpm -q seccomponent
    if [ $? -eq 0 ]; then
        echo "Seccomponent with older version is already installed. Uninstall it firstly."
        rpm -e seccomponent
        if [ $? -ne 0 ]; then
            echo "Failed to uninstall seccomponent with older version."
            return 1
        fi
    fi

    echo "Start to install ${rpmPath}."
    rpm -ivh "${rpmPath}"
    return $?
}

function decrypt_password() {
    local cipher="$1"
    local sccConf="$2"

    if [ -z "${cipher}" ]; then
        return $ERR_CODE_CIPHER_IS_EMPTY
    fi

    if [ ! -f "${sccConf}" ]; then
        return $ERR_CODE_SCC_CONF_FILE_NOT_EXIST
    fi

    export "SCC_CONF=${sccConf}"
    local passwd="$(echo "${cipher}" | ${DECRYPT_CMD})"
    local retVal="$?"
    passwd=$(echo "${passwd}" | awk -F ':' '{print $2}')
    if [ $retVal -ne 0 ] || [ -z "${passwd}" ]; then
        echo ""
        return $ERR_CODE_DECRYPT_PWD
    fi

    echo "${passwd}"
    return 0
}

function encrypt_password() {
    local passwd="$1"
    local sccConf="$2"

    if [ -z "${passwd}" ]; then
        return $ERR_CODE_CIPHER_IS_EMPTY
    fi

    if [ ! -f "${sccConf}" ]; then
        return $ERR_CODE_SCC_CONF_FILE_NOT_EXIST
    fi

    export "SCC_CONF=${sccConf}"
    local cipher="$(echo "${passwd}" | ${ENCRYPT_CMD})"
    local retVal="$?"
    cipher=$(echo "${cipher}" | awk -F ':' '{print $2}')
    if [ $retVal -ne 0 ] || [ -z "${cipher}" ]; then
        echo ""
        return $ERR_CODE_ENCRYPT_PWD
    fi

    echo "${cipher}"
    return 0
}

##############################################################
## @Usage : rand minNum maxNum
## @Return : a random number
## @Description : Generate a random number.
##############################################################
 function rand(){
    local min=$1
    local max=$(($2-$min+1))
    local num=$(cat /proc/sys/kernel/random/uuid | cksum | awk -F ' ' '{print $1}')
    echo $(($num%$max+$min))
}

##############################################################
## @Usage generate_random_pwd
## @Return  a random cipher
## @Description : Generate a random cipher.
##############################################################
function generate_random_pwd() {
    local len=${#BASE_CHARS}
    local randpwd=""
    for((i=0; i<=$PWD_LEN; i=i+1))
    do
        if [ $i -eq 3 ];then
            len=${#ALP_CHARS}
            let len=len-1
            rnd=$(rand 1 $len)
            randpwd="$randpwd"${ALP_CHARS:$rnd:1}
        elif [ $i -eq 6 ];then
            len=${#BASE_SPEC_CHARS}
            let len=len-1
            rnd=$(rand 1 $len)
            randpwd="$randpwd"${BASE_SPEC_CHARS:$rnd:1}
        elif [ $i -eq 9 ];then
            len=${#DIGIT_CHARS}
            let len=len-1
            rnd=$(rand 1 $len)
            randpwd="$randpwd"${DIGIT_CHARS:$rnd:1}
        else
            let len=len-1
            rnd=$(rand 1 $len)
            randpwd="$randpwd"${BASE_CHARS:$rnd:1}
        fi
    done

    echo "$randpwd"
}
