/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2018-2018. All rights reserved.
 * Description: 使用公有云平台统一的加解密可跨语言调用的接口,
 *              1)调用公司KMC组件实现密钥管理和AES对称加解密组件功能
 *              2)实现RSA非对称加解密功能
* Notes: 所有**二级指针参数，在函数调用成功后由调用者释放内存，但如果调用不成功则由实现函数负责释放内存
 * Author: f00178233
 * Create: 2018-10-08
 */

#ifndef SC_CRYPTOAPI_H
#define SC_CRYPTOAPI_H

#include "sc_type.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#ifdef SC_OS_WINDOW
#define SC_DLL_EXPORT  __declspec(dllexport)
#else
#define SC_DLL_EXPORT
#endif


/*****************************************************************************
 功能描述:读取配置文件初始化组件
 注意事项：加解密组件采用实现跟配置完全分离,不能采用组件的预置配置文件；
           配置文件和配置内容里的所有路径都采用服务路径，避免产生文件权限不足；
		   如初始化失败，可查看\include\errorcode.h来定位。并查看运行日志文件
		   如初始化运行日志前失败，可以通过执行 export SCC_LOG=/tmp/scc.log 设置初始化的日志文件
*****************************************************************************/
SC_DLL_EXPORT int SCC_Initialize(char *sccCfgFile);

/*****************************************************************************
 功能描述: 使用配置项初始化组件
 注意事项：配置内容里的所有路径都采用服务路径，避免产生文件权限不足；
*****************************************************************************/
SC_DLL_EXPORT int SCC_InitializeWithConfig(configuration *config);

/*****************************************************************************
 功能描述:组件结束函数，用于释放组件内部使用的内存和锁
 注意事项：需要跟Initialize成对出现
*****************************************************************************/
SC_DLL_EXPORT int SCC_Finalize();

/*****************************************************************************
 功能描述:不指定domain时的加解密，设置用默认domain 0
 注意事项：无
*****************************************************************************/
SC_DLL_EXPORT void SCC_ResetDomain();

/*****************************************************************************
 功能描述:不指定domain时的加解密，设置使用的domain
 注意事项：仅能用于在一个进程内不切换domain的使用场景，否则需要在加解密时指定domain
*****************************************************************************/
SC_DLL_EXPORT void SCC_SetDomain(int domainID);

/*****************************************************************************
 功能描述:加密函数，采用SetDomain设置/默认/指定的domain的Key进行加密
 注意事项:对频繁切换 domain的 加解密建议用EncryptByDomain
 入参:  algorithmId 指定加密算法进行加密
        domainID 指定domain的key进行加密
        plainText表示明文内存块
       plainTextLen表示明文长度
 出参: cipherText 表示将要生成加密后密文的内存块，二级指针，由调用者释放内存
        cipherTextLen表示加密后密文的实际长度
 返回值: SEC_SUCCESS: 成功
       其他: 失败
*****************************************************************************/
SC_DLL_EXPORT int SCC_Encrypt(const char *plainText, int plainTextLen, char **cipherText, int *cipherTextLen);

SC_DLL_EXPORT int
SCC_EncryptByDomain(int domainID, const char *plainText, int plainTextLen, char **cipherText, int *cipherTextLen);

SC_DLL_EXPORT int
SCC_EncryptByAlgorithm(int algorithmID, const char *plainText, int plainTextLen, char **cipherText, int *cipherTextLen);

SC_DLL_EXPORT int
SCC_EncryptByAlgorithmDomain(int algorithmID, int domainID, const char *plainText, int plainTextLen, char **cipherText,
                             int *cipherTextLen);

/*****************************************************************************
 功能描述:解密函数，采用SetDomain设置/默认/指定的domain的Key进行解密
 注意事项:对频繁切换 domain的 加解密建议用DecryptByDomain
 入参:  domainID 指定domain的key进行解密(KMC采用key Hash值确认密钥，实际未使用domainID)
        cipherText表示密文内存块
        cipherTextLen表示密文的申请长度
 出参: plainText 表示解密后明文内存块，由调用者释放内存
       plainTextLen表示解密后明文内存块的长度
 返回值:  SEC_SUCCESS: 成功
          其他: 失败
*****************************************************************************/
SC_DLL_EXPORT int SCC_Decrypt(const char *cipherText, int cipherTextLen, char **plainText, int *plainTextLen);

SC_DLL_EXPORT int
SCC_DecryptByDomain(int domainID, const char *cipherText, int cipherTextLen, char **plainText, int *plainTextLen);

/*****************************************************************************
 功能描述:解密后重新加密函数，用旧密钥解密后采用SetDomain设置/默认/指定的新key加密
 注意事项:对频繁切换domain的 加解密建议用ReEncryptByDomain
*****************************************************************************/
SC_DLL_EXPORT int SCC_ReEncrypt(const char *cipherText, int cipherTextLen, char **newCipherText, int *newCipherTextLen);

SC_DLL_EXPORT int SCC_ReEncryptByDomain(int domainID, const char *cipherText, int cipherTextLen, char **newCipherText,
                                        int *newCipherTextLen);

SC_DLL_EXPORT int
SCC_ReEncryptByAlgorithm(int algorithmID, const char *cipherText, int cipherTextLen, char **newCipherText,
                         int *newCipherTextLen);

SC_DLL_EXPORT int
SCC_ReEncryptByAlgorithmDomain(int algorithmID, int domainID, const char *cipherText, int cipherTextLen,
                               char **newCipherText,
                               int *newCipherTextLen);
/*****************************************************************************
 功能描述:将密钥keystore文件备份
 注意事项:无
*****************************************************************************/
SC_DLL_EXPORT int SCC_GenNewKeyFile(const char *toFile);

/*****************************************************************************
 功能描述:将keystone文件导出到指定文件，用于备份密钥文件
 注意事项:无
 入参: toFile表示导出的文件名
       pwd表示口令
        pwdLen表示口令长度
 出参: 无
 返回值: SEC_SUCCESS: 成功
         其他: 失败
*****************************************************************************/
SC_DLL_EXPORT int SCC_ExportKey(const char *toFile, const char *pwd, int pwdLen);
/*****************************************************************************
 功能描述: 从指定keystone文件导入MK数据，用于恢复密钥文件
 入参: fromFile表示要导入的keystone文件
       pwd表示口令
       pwdLen表示口令长度
 出参: 无
 返回值: SEC_SUCCESS: 成功
         其他: 失败
*****************************************************************************/
SC_DLL_EXPORT int SCC_ImportKey(const char *fromFile, const char *pwd, int pwdLen);

/*****************************************************************************
 功能描述:重新生成公私钥对，主要用于更换密钥管理
 注意事项:首次获取公钥时会自动生成公私钥
          当前版本采用覆盖旧密钥方式，所以公私钥不允许用于持久化加密
*****************************************************************************/
SC_DLL_EXPORT int SCC_ResetRSAKey();

/***************************************************************************************************************
 功能描述:实现非安全通道或弱认证通道的敏感信息传输
 注意事项:从远端获取密钥，在本地用远端加密后，然后把密文传递到远端用私钥解密
 ===============================================================================================================
 * Modified by l00403032: added identifier "const" to parameter "publicKey" of SCC_PublicKeyEncrypt funciton.
 * 2019-11-18
***************************************************************************************************************/
SC_DLL_EXPORT int SCC_GetPublicKey(char **key);

SC_DLL_EXPORT int
SCC_PublicKeyEncrypt(const char *plainText, const int plainTextLen, const char *publicKey, char **cipherText,
                     int *cipherTextLen);

SC_DLL_EXPORT int
SCC_PrivateKeyDecrypt(const char *cipherText, const int cipherTextLen, char **plainText, int *plainTextLen);

/*****************************************************************************
 功能描述:在指定的domain 激活新的密钥，
 注意事项:1.默认domain 0首次使用自动生成密钥，无需调用该函数
          2.非0采用内部申请key的domain首次使用需要调用该函数，可以初始化时调用GetMaxMkID判断是否存在key
		  3.所有内部管理Master Key的生成方式都采用KMC组件KMC_MK_GEN_BY_IMPORT类型模拟,未使用KMC_MK_GEN_BY_INNER类型
		  4.生成新的key,不会删除历史的key,Keystore文件中所有domain累计允许存储MK最大数量为4096,要规划好激活频度
*****************************************************************************/
SC_DLL_EXPORT int SCC_ActiveNewKey(int domainID);

/*****************************************************************************
 功能描述:删除domain下所有非激活的key，用于key数量超过最大值紧急场景，仅限命令行调用
 注意事项:1. Keystore文件中所有domain累计允许存储MK最大数量为4096
          2. 业务禁止直接调用该函数
*****************************************************************************/
SC_DLL_EXPORT int SCC_ClearInactiveKey(int domainID);

SC_DLL_EXPORT int SCC_GenSecKey(int keyLen, char **key, int *keyBase64Len);

SC_DLL_EXPORT int SCC_GenSecByteKey(int keyLen, char **key);

/*****************************************************************************
 功能描述:在指定的domain 外部注册新的密钥
 注意事项:1.默认domain 0为自动生成密钥，无法调用该函数注册
          2.非0的domain如采用外部注册key,首次加密前需要调用该函数，
		  3.keyID输入0表示自动获取最大的KeyID+1作为新的KeyID
		  4.初始化时调用GetMaxMkID判断是否存在key，调用IsExistMkID判断是否存在keyID避免keyID已存在
		  5.所有内部管理Master Key的生成方式都采用KMC组件KMC_MK_GEN_BY_IMPORT类型模拟,未使用KMC_MK_GEN_BY_INNER类型
		  6.生成新的key,不会删除历史的key,Keystore文件中所有domain累计允许存储MK最大数量为4096,要规划好激活频度
 ==============================================================================================================================
 * Modified by l00403032: added identifier "const" to parameter "base64Key" and "key" of SCC_RegisterKey and SCC_RegisterByteKey funcitons.
 * 2019-11-18
*****************************************************************************/
SC_DLL_EXPORT int SCC_RegisterKey(int domainID, int keyID, const char *base64Key, int base64KeyLen);

SC_DLL_EXPORT int SCC_RegisterByteKey(int domainID, int keyID, const char *key, int keyLen);

SC_DLL_EXPORT int SCC_RegisterByteKeyEx(int domainID, int keyID, const char* key, int keyLen);

/*****************************************************************************
 功能描述:获取指定domain和key ID的密钥，用于外部进行加解密，实现扩展功能
 注意事项:1.需要做好密钥的敏感信息保护
          2. keyID<=0时，返回当前domain Active Key
*****************************************************************************/
SC_DLL_EXPORT int SCC_GetKeyByID(int domainID, int keyID, char **key, int *keyLen, int base64Ind);

SC_DLL_EXPORT int SCC_GetKeyByIDEx(int domainID, int keyID, char **key, int *keyLen, int base64Ind);

/*****************************************************************************
 功能描述:用于定位问题，查看Keystore内部domain,RootKey,MasterKey的情况
 注意事项:1.输出到运行日志文件
*****************************************************************************/
SC_DLL_EXPORT void SCC_PrintAllInfoToLog();

/*****************************************************************************
 功能描述:获取指定domain当前最大的keyID,用于判断该domain是否初始化过key
 注意事项:maxKeyID返回为0 表示domain未初始化key
*****************************************************************************/
SC_DLL_EXPORT int SCC_GetMaxMkID(int domainID, int *maxKeyID);

SC_DLL_EXPORT int SCC_GetMaxMkIDEx(int domainID, int *maxKeyID);

/*****************************************************************************
 功能描述:判断指定domain是否存在指定的keyID,用于避免重复导入key.
 注意事项:result返回 type.h头文件定义的SEC_FALSE(0)表示不存在，返回SEC_TRUE (1)表示存在
*****************************************************************************/
SC_DLL_EXPORT int SCC_IsExistMkID(int domainID, int keyID, int *result);

SC_DLL_EXPORT int SCC_IsExistMkIDEx(int domainID, int keyID, int *result);

/*****************************************************************************
 功能描述: 当根密钥过期时用于更新根密钥，对多进程复用同一个Key文件的场景慎用。
*****************************************************************************/
SC_DLL_EXPORT int SCC_UpdateRootKey();

/*****************************************************************************
 功能描述:  慎用，将主备Keystore文件彻底删除，会使用空值复写keystore文件，然后删除文件。
 删除后keystore无法复原，内存中的密钥也会删除。使用该函数的场景时：需要销毁所有密钥的时候。
*****************************************************************************/
SC_DLL_EXPORT int SCC_SecureEraseKeystore();

/*****************************************************************************
 功能描述:  释放内存。解决Windows下的c库未提供的free接口
*****************************************************************************/
SC_DLL_EXPORT void SCC_Free(void *ptr);

/*****************************************************************************
 功能描述: 外部接口函数，使用指定算法进行hash
            调用成功后hashCode需要调用者释放
*****************************************************************************/
SC_DLL_EXPORT int SCC_CreateHashCode(int algorithmId, const char *plainText, int plainTextLen, char **hashCode, int *hashCodeLen);

/*****************************************************************************
 功能描述: 外部接口函数，使用指定算法验证hashcode
*****************************************************************************/
SC_DLL_EXPORT int SCC_CheckHashCode(int algorithmId, const char *plainText, int plainTextLen, const char *cmpHashCode, int hashCodeLen, int *checkRet);

/*****************************************************************************
 功能描述: 外部接口函数，使用指定算法和domain的key进行hmac hash
            调用成功后hmacData需要调用者释放
*****************************************************************************/
SC_DLL_EXPORT int SCC_CreateHmacCode(int algorithmId, int domain, const char *plainText, int plainTextLen, char **hmacData, int *hmacDataLen);

/*****************************************************************************
 功能描述: 外部接口函数，使用指定domain的key验证hmac hashcode
*****************************************************************************/
SC_DLL_EXPORT int SCC_CheckHmacCode(int domain, const char *plainText, int plainTextLen, const char *hmacData, int hmacDataLen, int *checkRet);

/*****************************************************************************
 功能描述: 外部接口函数，使用指定算法和domain的key对文件进行hmac hash
            调用成功后hmacData需要调用者释放
*****************************************************************************/
SC_DLL_EXPORT int SCC_CreateFileHmacCode(int algorithmId, int domain, const char *file, char **hmacData, int *hmacDataLen);

/*****************************************************************************
 功能描述: 外部接口函数，使用指定domain的key验证文件hashcode
*****************************************************************************/
SC_DLL_EXPORT int SCC_CheckFileHmacCode(int domain, const char *file, const char *hmacData, int hmacDataLen, int *checkRet);

/*****************************************************************************
 功能描述: 外部接口函数，使用指定算法对密码进行hash保护
            调用成功后cipherText需要调用者释放
*****************************************************************************/
SC_DLL_EXPORT int SCC_ProtectPwd(int algorithmId, const char *plainText, int plainTextLen, char **cipherText, int *cipherTextLen);

/*****************************************************************************
 功能描述: 外部接口函数，验证密码的hashcode
*****************************************************************************/
SC_DLL_EXPORT int SCC_VerifyPwd(const char *plainText, int plainTextLen, const char *cipherText, int cipherTextLen, int *checkRet);

/*****************************************************************************
 功能描述: 外部接口函数，创建指定domain
*****************************************************************************/
SC_DLL_EXPORT int SCC_CreateDomain(const int domainId);

/*****************************************************************************
 功能描述: 外部接口函数，删除指定domain
*****************************************************************************/
int SCC_DeleteDomain(const int domainId);

/*****************************************************************************
 功能描述: 外部接口函数，导出指定domain的密钥
            调用成功后key需要调用者释放
*****************************************************************************/
SC_DLL_EXPORT int SCC_ExportKeyByDomain(const int domainId, int isHw, char **key, int *keyLen);

/*****************************************************************************
 功能描述: 外部接口函数，导入指定domain的密钥，追加添加
*****************************************************************************/
SC_DLL_EXPORT int SCC_ImportKeyByDomain(const int domainId, int isHw, int isEncrypt, const char *key, int keyLen);

SC_DLL_EXPORT int SCC_CheckHwKey(const char *cipherText, int cipherTextLen, int *checkRet);

SC_DLL_EXPORT int SCC_GetCipherAlg(const char *cipherText, int cipherTextLen, int *AlgId);

/**************************************************************************
  功能描述: 外部接口函数，更新 ak/sk
***************************************************************************/
SC_DLL_EXPORT int SCC_UpdateAccessKey(const char *ak, const int akLen, const char *sk, const int skLen);

/*****************************************************************************
 功能描述:默认domain为0
*****************************************************************************/
#define DEFAULT_DOMAIN 0
/*****************************************************************************
 功能描述:32byte表示256bit
*****************************************************************************/
#define KEY_DEFAULT_LEN 32
#define KEY_MIN_LEN 16
#define KEYLIFE_DEFAULT_DAYS 180

#ifdef __cplusplus
}
#endif  /* __cplusplus */

#endif
